@@ -2,7 +2,6 @@
 use warnings;
 use strict;
 
-use Data::Dumper;
 use Devel::Examine::Subs;
 
 my $ver = $ARGV[0] || die "\nsupply a version string\n";
@@ -13,11 +12,7 @@ my $des = Devel::Examine::Subs->new(
                         );
 
 my $cref = sub { 
-        # this is the v5.12 way
-        #$_[0] =~ s/^(package\s+\w+(?:::\w+)*)\s+\d+\.\d{2}/$1 $ver/; 
-        $_[0] =~ s/^(our\s+\$VERSION\s+=\s+')/$1 $ver\';/; 
-        
+        $_[0] =~ s/^(package\s+\w+(?:::\w+)*)\s+\d+\.\d{2}/$1 $ver/; 
     };
 
 my $ret = $des->replace(exec => $cref, limit => 1);
-print Dumper $ret;
