@@ -1,4 +1,4 @@
-package Devel::Examine::Subs 1.41;
+package Devel::Examine::Subs 1.43;
 use 5.012;
 use warnings;
 use strict;
@@ -29,6 +29,10 @@ BEGIN {
     }
 };
 
+#
+# public methods
+#
+
 sub new {
    
     # set up for tracing
@@ -78,6 +82,278 @@ sub run {
 
     return $struct;
 }
+sub has {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $p = $self->_params(@_);
+
+    $self->{params}{pre_filter} = 'file_lines_contain';
+    $self->{params}{engine} = 'has';
+    
+    $self->run($p);
+}
+sub missing {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $p = $self->_params(@_);
+
+    $self->{params}{engine} = 'missing';
+    
+    $self->run($p);
+}
+sub all {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $p = $self->_params(@_);
+
+    $self->{params}{engine} = 'all';
+    
+    $self->run($p);
+}
+sub lines {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $p = $self->_params(@_);
+
+    $self->{params}{engine} = 'lines';
+    
+    if ($self->{params}{search} || $p->{search}){
+        $self->{params}{pre_filter} = 'file_lines_contain';
+    }
+
+    $self->run($p);
+}
+sub module {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $p = $self->_params(@_);
+
+    # set the preprocessor up, and have it return before
+    # the building/compiling of file data happens
+
+    $self->{params}{pre_proc} = 'module';
+    $self->{params}{pre_proc_return} = 1;
+
+    $self->{params}{engine} = 'module';
+
+    $self->run($p);
+}
+sub objects {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $p = $self->_params(@_);
+
+    $self->{params}{pre_filter} = 'subs';
+    $self->{params}{engine} = 'objects';
+
+    $self->run($p);
+}
+sub search_replace {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $p = $self->_params(@_);
+
+    $self->{params}{pre_filter}
+      = ['file_lines_contain', 'subs', 'objects'];
+
+    $self->{params}{engine} = 'search_replace';
+
+    $self->run($p);
+}
+sub inject_after {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $p = $self->_params(@_);
+
+    if (! $p->{injects} && ! $self->{params}{injects}){
+        $p->{injects} = 1;
+    }
+
+    $self->{params}{pre_filter}
+      = ['file_lines_contain', 'subs', 'objects'];
+
+    $self->{params}{engine} = 'inject_after';
+
+    $self->run($p);
+}
+sub inject {
+    
+    trace() if $ENV{TRACE};
+    my $self = shift;
+    my $p = $self->_params(@_);
+
+    # inject_use/inject_after_sub_def are preprocs
+
+    if ($p->{inject_use} || $p->{inject_after_sub_def}){
+        $self->{params}{pre_proc} = 'inject';
+        $self->{params}{pre_proc_return} = 1;
+    }
+
+    $self->run($p);
+}
+sub replace {
+
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $p = $self->_params(@_);
+
+    $self->{params}{pre_proc} = 'replace';
+    $self->{params}{pre_proc_return} = 1;
+
+    $self->run($p);
+}
+sub remove {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $p = $self->_params(@_);
+
+    $self->{params}{pre_proc} = 'remove';
+    $self->{params}{pre_proc_return} = 1;
+
+    $self->run($p);
+}
+
+#
+# publicly available private methods
+#
+
+sub add_functionality {
+   
+    trace() if $ENV{TRACE};
+    
+    my $self = shift;
+    my $p = $self->_params(@_);
+
+    $self->_config($p);
+    
+    my $to_add = $self->{params}{add_functionality};
+    
+    my $in_prod = $self->{params}{add_functionality_prod};
+
+    my @allowed = qw(
+                    pre_proc
+                    pre_filter
+                    engine
+    );
+
+
+    if (! (grep {$to_add eq $_} @allowed)){
+        croak "Adding a non-allowed piece of functionality...\n";
+    }
+
+    my %dt = (
+            engine => sub {
+                        trace() if $ENV{TRACE};
+                        return $in_prod
+                        ? $INC{'Devel/Examine/Subs/Engine.pm'}
+                        : 'lib/Devel/Examine/Subs/Engine.pm';
+                      },
+    );
+
+    my $caller = (caller)[1];
+
+    open my $fh, '<', $caller
+      or confess "can't open the caller file $caller: $!";
+
+    my $code_found = 0;
+    my @code;
+
+    while (<$fh>){
+        chomp;
+        if (m|^#(.*)<des>|){
+            $code_found = 1;
+            next;
+        }
+        next if ! $code_found;
+        last if m|^#(.*)</des>|;
+        push @code, $_;
+    }
+
+    my $file = $dt{$to_add}->();
+
+    my $des = Devel::Examine::Subs->new({
+                                    file => $file,
+                                    pre_filter => 'end_of_last_sub',
+                                });
+
+    $p->{write_file_contents} = \@code;
+}
+sub pre_procs {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $module = $self->{namespace} . "::Preprocessor";
+    my $pre_proc = $module->new;
+
+    my @pre_procs;
+
+    for (keys %{$pre_proc->_dt}){
+        push @pre_procs, $_ if $_ !~ /^_/;
+    }
+    return @pre_procs;
+}
+sub pre_filters {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $module = $self->{namespace} . "::Prefilter";
+    my $pre_filter = $module->new;
+
+    my @pre_filters;
+
+    for (keys %{$pre_filter->_dt}){
+        push @pre_filters, $_ if $_ !~ /^_/;
+    }
+    return @pre_filters;
+}
+sub engines {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $module = $self->{namespace} . "::Engine";
+    my $engine = $module->new;
+ 
+    my @engines;
+
+    for (keys %{$engine->_dt}){
+        push @engines, $_ if $_ !~ /^_/;
+    }
+    return @engines;
+}
+sub valid_params {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    return %{$self->{valid_params}};
+}
+
+#
+# private methods
+#
+
 sub _run_directory {
     
     trace() if $ENV{TRACE};
@@ -239,6 +515,8 @@ sub _config {
         inject_after_sub_def => 0,
         delete => 0,
         file_contents => 0,
+        exec => 0,
+        limit => 0,
     );
 
     $self->{valid_params} = \%valid_params;
@@ -362,10 +640,10 @@ sub _file {
         
         if (! $INC{$file}){
             
-            eval { use Data::Dump; };
+            eval { require $file; import $module; };
 
             if ($@){
-                $@ = "Devel::Examine::Subs::_file() speaking ... " .
+                $@ = "\nDevel::Examine::Subs::_file() speaking ... " .
                      "Can't transform module to a file name\n\n"
                      . $@;
                 croak $@;
@@ -822,264 +1100,19 @@ sub _engine {
 
     return $cref;
 }
-sub pre_procs {
-    
-    trace() if $ENV{TRACE};
 
-    my $self = shift;
-    my $module = $self->{namespace} . "::Preprocessor";
-    my $pre_proc = $module->new;
+#
+# pod
+#
 
-    my @pre_procs;
-
-    for (keys %{$pre_proc->_dt}){
-        push @pre_procs, $_ if $_ !~ /^_/;
-    }
-    return @pre_procs;
-}
-sub pre_filters {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $module = $self->{namespace} . "::Prefilter";
-    my $pre_filter = $module->new;
-
-    my @pre_filters;
-
-    for (keys %{$pre_filter->_dt}){
-        push @pre_filters, $_ if $_ !~ /^_/;
-    }
-    return @pre_filters;
-}
-sub engines {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $module = $self->{namespace} . "::Engine";
-    my $engine = $module->new;
- 
-    my @engines;
-
-    for (keys %{$engine->_dt}){
-        push @engines, $_ if $_ !~ /^_/;
-    }
-    return @engines;
-}
-sub has {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $p = $self->_params(@_);
-
-    $self->{params}{pre_filter} = 'file_lines_contain';
-    $self->{params}{engine} = 'has';
-    
-    $self->run($p);
-}
-sub missing {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $p = $self->_params(@_);
-
-    $self->{params}{engine} = 'missing';
-    
-    $self->run($p);
-}
-sub all {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $p = $self->_params(@_);
-
-    $self->{params}{engine} = 'all';
-    
-    $self->run($p);
-}
-sub lines {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $p = $self->_params(@_);
-
-    $self->{params}{engine} = 'lines';
-    
-    if ($self->{params}{search} || $p->{search}){
-        $self->{params}{pre_filter} = 'file_lines_contain';
-    }
-
-    $self->run($p);
-}
-sub module {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $p = $self->_params(@_);
-
-    # set the preprocessor up, and have it return before
-    # the building/compiling of file data happens
-
-    $self->{params}{pre_proc} = 'module';
-    $self->{params}{pre_proc_return} = 1;
-
-    $self->{params}{engine} = 'module';
-
-    $self->run($p);
-}
-sub objects {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $p = $self->_params(@_);
-
-    $self->{params}{pre_filter} = 'subs';
-    $self->{params}{engine} = 'objects';
-
-    $self->run($p);
-}
-sub search_replace {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $p = $self->_params(@_);
-
-    $self->{params}{pre_filter}
-      = ['file_lines_contain', 'subs', 'objects'];
-
-    $self->{params}{engine} = 'search_replace';
-
-    $self->run($p);
-}
-sub inject {
-    
-    trace() if $ENV{TRACE};
-    my $self = shift;
-    my $p = $self->_params(@_);
-
-    # inject_use/inject_after_sub_def are preprocs
-
-    if ($p->{inject_use} || $p->{inject_after_sub_def}){
-        $self->{params}{pre_proc} = 'inject';
-        $self->{params}{pre_proc_return} = 1;
-    }
-
-    $self->run($p);
-}
-sub remove {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $p = $self->_params(@_);
-
-    $self->{params}{pre_proc} = 'remove';
-    $self->{params}{pre_proc_return} = 1;
-
-    $self->run($p);
-}
-sub inject_after {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $p = $self->_params(@_);
-
-    if (! $p->{injects} && ! $self->{params}{injects}){
-        $p->{injects} = 1;
-    }
-
-    $self->{params}{pre_filter}
-      = ['file_lines_contain', 'subs', 'objects'];
-
-    $self->{params}{engine} = 'inject_after';
-
-    $self->run($p);
-}
-sub add_functionality {
-   
-    trace() if $ENV{TRACE};
-    
-    my $self = shift;
-    my $p = $self->_params(@_);
-
-    $self->_config($p);
-    
-    my $to_add = $self->{params}{add_functionality};
-    
-    my $in_prod = $self->{params}{add_functionality_prod};
-
-    my @allowed = qw(
-                    pre_proc
-                    pre_filter
-                    engine
-    );
-
-
-    if (! (grep {$to_add eq $_} @allowed)){
-        croak "Adding a non-allowed piece of functionality...\n";
-    }
-
-    my %dt = (
-            engine => sub {
-                        trace() if $ENV{TRACE};
-                        return $in_prod
-                        ? $INC{'Devel/Examine/Subs/Engine.pm'}
-                        : 'lib/Devel/Examine/Subs/Engine.pm';
-                      },
-    );
-
-    my $caller = (caller)[1];
-
-    open my $fh, '<', $caller
-      or confess "can't open the caller file $caller: $!";
-
-    my $code_found = 0;
-    my @code;
-
-    while (<$fh>){
-        chomp;
-        if (m|^#(.*)<des>|){
-            $code_found = 1;
-            next;
-        }
-        next if ! $code_found;
-        last if m|^#(.*)</des>|;
-        push @code, $_;
-    }
-
-    my $file = $dt{$to_add}->();
-
-    my $des = Devel::Examine::Subs->new({
-                                    file => $file,
-                                    pre_filter => 'end_of_last_sub',
-                                });
-
-    $p->{write_file_contents} = \@code;
-}
-sub valid_params {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    return %{$self->{valid_params}};
-}
 sub _pod{} #vim placeholder 
 1; 
 __END__
 
 =head1 NAME
 
-Devel::Examine::Subs - Get info, search/replace and inject code in
-Perl file subs.
+Devel::Examine::Subs - Get info about, search/replace and inject code into
+Perl files and subs.
 
 =head1 SYNOPSIS
 
@@ -1102,7 +1135,6 @@ Get all the subs as objects
         $sub->line_count; # number of lines in sub
         $sub->code;       # entire sub code from file
         $sub->lines;      # lines that match search term
-
     }
 
 Get the sub objects within a hash
@@ -1131,10 +1163,7 @@ Get all subs containing "string" in the body
 
 Search and replace code in subs
 
-    $des->search_replace(
-                    search => q/\$template = 'one\.tmpl'",
-                    replace => '$template = \'two.tmpl\'',
-                  );
+    $des->search_replace( exec => sub { $_[0] =~ s/this/that/g; } );
 
 Inject code into sub after a search term (preserves previous line's indenting)
 
@@ -1306,15 +1335,13 @@ an array reference which contains a hash reference in the format line_number
 
 =head2 C<search_replace>
 
-Mandatory parameters: C<search =E<gt> 'this', replace =E<gt> 'that'>
+Mandatory parameters: C<exec =E<gt> $cref>
 
 Core optional parameter: C<copy =E<gt> 'filename.txt'>
 
-Search for lines that contain certain text, and replace the search term with
-the replace term. If the optional parameter 'copy' is sent in, a copy of the
-original file will be created in the current directory with the name
-specified, and that file will be worked on instead. Good for testing to ensure
-The Right Thing will happen in a production file.
+Coderef should be created in the form C<sub { $_[0] =~ s/search/replace/; };>.
+This allows us to avoid string C<eval>, and allows us to use any regex
+modifiers you choose.
 
 This method will create a backup copy of the file with the same name appended
 with '.bak', but don't confuse this feature with the 'copy' parameter.
@@ -1350,6 +1377,20 @@ This method is file based... the work happens prior to digging up subs, hence
 C<exclude>, C<include> and other sub-based parameters have no effect.
 
 
+=head2 C<replace>
+
+Parameters: C<exec =E<gt> $cref, limit =E<gt> 1>
+
+This is the entire file brother to the sub-only C<search_replace()>. The C<limit> parameter
+specifies how many successful replacements to do, starting at the top of the file. Set to a
+negative integer for unlimited (this is the default).
+
+The C<exec> parameter is a code reference, eg: C<my $cref = sub {$_[0] =~ s/this/that/;}>.
+All standard Perl regular expressions apply, along with their modifiers.
+
+Returns the number of lines changed in file mode, and an empty hashref in directory mode.
+
+
 
 =head2 C<inject_after>
 
